#include "Element.h"
#include "Class.h"


namespace Cgi
{


Element::Element(const std::string& tag)
: m_tag(tag)
, m_parent(NULL)
, m_class(NULL)
{
}


Element::~Element()
{
}


void Element::SetClass(Class *cl)
{
	m_class = cl;
}


Class *Element::GetClass() const
{
	return m_class;
}


void Element::SetId(const std::string& id)
{
	m_id = id;
}


const std::string& Element::GetId() const
{
	return m_id;
}


void Element::SetStyle(const std::string& style)
{
	m_style = style;
}


const std::string& Element::GetStyle() const
{
	return m_style;
}


void Element::SetParent(Element *parent)
{
	m_parent = parent;
}


Element *Element::GetParent() const
{
	return m_parent;
}


std::string Element::GetAttributes(const std::string& tag) const
{
	std::string r;
	if (tag.size())
	{
		r += "<" + tag;
	}
	if (m_class)
	{
		// %! htmlsafestr ( m_class -> GetName() )
		r += " class='" + m_class -> GetName() + "'";
	}
	if (m_id.size())
	{
		// %! htmlsafestr ( m_id )
		r += " id='" + m_id + "'";
	}
	if (m_style.size())
	{
		// %! htmlsafestr ( m_style )
		r += " style='" + m_style + "'";
	}
	for (attrmap_t::const_iterator it = m_attrs.begin(); it != m_attrs.end(); it++)
	{
		// %! htmlsafestr ( it -> second )
		r += " " + it -> first + "='" + it -> second + "'";
	}
	if (tag.size())
	{
		r += ">";
	}
	return r;
}


void Element::SetAttribute(const std::string& key, const std::string& value)
{
	m_attrs[key] = value;
}


const std::string& Element::GetAttribute(const std::string& key)
{
	if (m_attrs.find(key) != m_attrs.end())
	{
		return m_attrs[key];
	}
	return m_temporary;
}


void Element::SetContent(const std::string& content)
{
	m_content = content;
}


const std::string& Element::GetContent() const
{
	return m_content;
}


std::string Element::ToString() const
{
	return GetAttributes( GetElementName() ) + GetContent() + "</" + GetElementName() + ">";
}


doctype_t Element::GetDoctype()
{
	return m_parent -> GetDoctype();
}


const std::string& Element::GetElementName() const
{
	return m_tag;
}


} // namespace Cgi
