#ifndef _CGI_ELEMENT_H
#define _CGI_ELEMENT_H

#include <string>
#include <map>

namespace Cgi
{

class Class;

typedef std::map<std::string, std::string> attrmap_t;

typedef enum {
	DOCTYPE_HTML_20 = 1,
	DOCTYPE_HTML_32,
	DOCTYPE_HTML_401_TRANSITIONAL,
	DOCTYPE_HTML_401_STRICT,
	DOCTYPE_HTML_401_FRAMESET,
	DOCTYPE_XHTML_1_0_TRANSITIONAL,
	DOCTYPE_XHTML_1_0_STRICT,
	DOCTYPE_XHTML_1_0_FRAMESET
} doctype_t;

class Element
{
public:
	Element(const std::string& tag);
	virtual ~Element();

	/** Generate full tag including content and closing tag. */
	virtual std::string ToString() const;

	virtual void SetClass(Class *cl);
	Class *GetClass() const;

	virtual void SetId(const std::string& id);
	const std::string& GetId() const;

	virtual void SetStyle(const std::string& style);
	const std::string& GetStyle() const;

	virtual void SetParent(Element *parent);
	Element *GetParent() const;

	virtual Element *Copy() = 0;

	/** Get class, id, style, and other attributes.
		\param tag Element name */
	std::string GetAttributes(const std::string& tag = "") const;

	const std::string& GetElementName() const;

	void SetAttribute(const std::string& key, const std::string& value);
	const std::string& GetAttribute(const std::string& key);

	void SetContent(const std::string& content);
	const std::string& GetContent() const;

	/** Get Document type from HtmlDocument. */
	virtual doctype_t GetDoctype();

private:
	std::string m_tag; ///< element name
	Element *m_parent;
	Class *m_class;
	std::string m_id;
	std::string m_style;
	attrmap_t m_attrs;
	std::string m_content;
	std::string m_temporary;
};


} // namespace Cgi


#endif // _CGI_ELEMENT_H
