#include "ElementList.h"
#include "RefCount.h"


namespace Cgi
{


ElementList::ElementList(const std::string& tag) : Element(tag)
, m_ref_count( new RefCount() )
{
}


ElementList::ElementList(const ElementList& ref) : Element(ref)
, m_ref_count( ref.m_ref_count )
{
	for (element_v::const_iterator it = ref.m_elements.begin(); it != ref.m_elements.end(); it++)
	{
		Element *p = *it;
		p -> SetParent( this );
		m_elements.push_back( p );
	}
	m_ref_count -> Increase();
}


ElementList::~ElementList()
{
	if (!m_ref_count -> Decrease())
	{
		for (element_v::const_iterator it = m_elements.begin(); it != m_elements.end(); it++)
		{
			Element *p = *it;
			delete p;
		}
		delete m_ref_count;
	}
}


void ElementList::Add(Element& el)
{
	el.SetParent( this );
	m_elements.push_back( el.Copy() );
}


std::string ElementList::GetElements() const
{
	std::string str;
	for (element_v::const_iterator it = m_elements.begin(); it != m_elements.end(); it++)
	{
		Element *p = *it;
		std::string tmp = p -> ToString();
		str += tmp;
	}
	return str;
}


std::string ElementList::ToString() const
{
	return GetAttributes( GetElementName() ) + GetElements() + "</" + GetElementName() + ">";
}


} // namespace Cgi


