// MenuWeb.h
/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _MENUWEB_H
#define _MENUWEB_H

#ifdef _WIN32
#define STL_USING_VECTOR
#include <stlhelper.h>
#else
#include <vector>
#endif

namespace Cgi {

//! Menuitems

enum
{
	MENUITEM_TYPE_FORM = 1,
	MENUITEM_TYPE_SEPARATOR,
	MENUITEM_TYPE_URL,
	MENUITEM_TYPE_IMAGE
};


//! Menu styles enumerated

enum 
{
	MENU_STYLE_FRAME_LEFT = 1,
	MENU_STYLE_TOP,
	MENU_STYLE_LEFT
};


//! Menu item definition struct

typedef struct
{
	int num;
	int type;
	WebForm *form;
	std::string title;
	std::string url; // <a href=...>
	std::string src; // <img src=...>
} MENUITEM;
typedef std::vector<MENUITEM *> menuitem_v;


//! Automatic menu creation functionality added to Web class

class MenuWeb : public Web
{
public:
	MenuWeb(int = MENU_STYLE_TOP);
	MenuWeb(const std::string &,const std::string &,int = MENU_STYLE_TOP);
	MenuWeb(int argc,char *argv[],const std::string &,const std::string &,int = MENU_STYLE_TOP);
	~MenuWeb();

	void SetMenuStyle(int);

	const MENUITEM & AddMenuItem(WebForm *,const std::string &title);
	void AddMenuSeparator();
	void AddMenuLink(const std::string &url,const std::string &title);
	void AddMenuImage(const std::string &src);

	virtual void ReadCookies();
	virtual void FormInput();
	virtual void CreateHeader();
	virtual void GenerateDocument();

	std::string GetCookiename() { return m_cookiename; }
	void SetCookiename(const std::string &str) { m_cookiename = str; }
	void SetPage(int x) { m_page = x; m_bSetCookie = true; }
	int GetPage() { return m_page; }
	void SetMenuclass(const std::string &x) { m_strClass = x; }

	void SetTheCookie();

private:
  MenuWeb(const MenuWeb& ) {}
  MenuWeb& operator=(const MenuWeb& ) { return *this; }
	int m_count;
	int m_style;
	int m_page;
	std::string m_cookiename;
	bool m_bSetCookie;
	menuitem_v m_items;
	std::string m_strClass;
};

} // namespace

#endif // _MENUWEB_H
