/*
 **	\file Select.h
 **	\date  2006-05-05
 **	\author grymse@alhem.net
**/
/*
Copyright (C) 2006  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef _CGIPP_SELECT_H
#define _CGIPP_SELECT_H

#include <string>
#include <list>
#include "Styled.h"


namespace Cgi {

class Select : public Styled
{
	class Value {
	public:
		Value(Select& sel,const std::string& text) : m_sel(sel), m_text(text) {}
		virtual ~Value() {}
		virtual void Print() {
			printf("<option%s>%s", Selected() ? " selected" : "", m_text.c_str());
		}
		virtual bool Selected() {
			if (!strcmp(m_text.c_str(), m_sel.m_selected_str.c_str()))
				return true;
			return false;
		}
		Select& m_sel;
		std::string m_text;
	};
	class Multi : public Value {
	public:
		Multi(Select& sel,long value,const std::string& text) : Value(sel, text), m_value(value) {}
		void Print() {
			printf("<option value=%ld%s>%s", m_value, Selected() ? " selected" : "", m_text.c_str());
		}
		bool Selected() {
			if (m_value == m_sel.m_selected)
				return true;
			return false;
		}
		long m_value;
	};
	friend class Value;
public:
	Select(const std::string& name);
	~Select();

	void Add(const std::string& value);
	void Add(long value,const std::string& description);

	void Selected(const std::string x) { m_selected_str = x; }
	void Selected(long x) { m_selected = x; }

	void Print();

private:
	Select(const Select& ) {} // copy constructor
	Select& operator=(const Select& ) { return *this; } // assignment operator
	std::string m_name;
	std::list<Value *> m_values;
	long m_selected;
	std::string m_selected_str;
};

} // namespace


#endif // _CGIPP_SELECT_H
